from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Audiences(Client):
    """
    Audience Discovery API
    """
    @sp_endpoint('/audiences/taxonomy/list', method='POST')
    def list_audiences_taxonomy(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/audiences/list', method='POST')
    def list_audiences(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Metadata(Client):

    @sp_endpoint('/product/metadata', method='POST')
    def get_products_metadata(self, **kwargs) -> ApiResponse:
        r"""

        get_products_metadata(self, **kwargs) -> ApiResponse

        Returns product metadata for the advertiser.

        body: | REQUIRED

            '**asins**': *list>string*, {'description': 'Specific asins to search for in the advertiser's inventory. Cannot use together with skus or searchStr input types.'}

            '**checkItemDetails**': *boolean*, {'description': 'Whether item details such as name, image, and price is required. default: false'}

            '**cursorToken**': *string*, {'description': 'Pagination token used for the suggested sort type'}

            '**adType**': *string*, {'description': 'Program type. Required if checks advertising eligibility. Enum: [ SP, SB, SD ]'}

            '**skus**': *list>string*, {'description': 'Specific skus to search for in the advertiser's inventory. Currently only support SP program type for sellers. Cannot use together with asins or searchStr input types'}

            '**checkEligibility**': *boolean*, {'description': 'Whether advertising eligibility info is required. default: false'}

            '**searchStr**': *string*, {'description': 'Specific string in the item title to search for in the advertiser's inventory. Case insensitive. Cannot use together with asins or skus input types'}

            '**pageIndex**': *integer($int32)*, {'description*': 'Index of the page to be returned'}

            '**sortOrder**': *string*, {'description': 'Sort order (has to be DESC for the suggested sort type). default: DESC. Enum [ ASC, DESC ]'}

            '**pageSize**': *integer($int32)*, {'description*': 'Number of items to be returned on this page index (max 100 for author)'}

            '**sortBy**': *string*, {'description': 'Sort option for the result. Currently only support SP program type for sellers. Enum [ SUGGESTED, CREATED_DATE ]'}



        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class TargetsRecommendations(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/recommendations/targets/product/list', method='POST')
    def list_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        Gets a list of recommended categories for targeting.

        Recommendations are based on the ASINs that are passed in the request.

        Request Body
            | '**nextToken**': *string*, {'description': 'Operations that return paginated results include a pagination token in this field. To retrieve the next page of results, call the same operation and specify this token in the request. If the NextToken field is empty, there are no further results.'}
            | '**maxResults**': *integer*, {'description': 'Sets a limit on the number of results returned by an operation. minimum: 1, maximum: 100'}
            | '**filters**': *SBExpression*, {'filterType': '[ ASINS ]', 'values': 'A list of ASINs / An ASIN.'}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/recommendations/targets/category', method='POST')
    def list_category_targets(self, **kwargs) -> ApiResponse:
        r"""
        Gets a list of recommended categories for targeting.

        Recommendations are based on the ASINs that are passed in the request.

        Request Body
            | '**asins**': *string*, {'description': 'A list of ASINs.'}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/recommendations/targets/brand', method='POST')
    def list_brand_targets(self, **kwargs) -> ApiResponse:
        r"""
        Gets a list of brand suggestions.

        The Brand suggestions are based on a list of either category identifiers or keywords passed in the request. It is not valid to specify both category identifiers and keywords in the request.

        Request Body (oneOf ->)
            | '**categoryId**': *integer($int64)*, {'description': 'The category identifier for which to get recommendations.'}
            | '**keyword**': *string*, {'description': 'The keyword for which to get recommendations.}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class NegativeTargets(Client):

    @sp_endpoint('/v2/sp/negativeTargets', method='POST')
    def create_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        create_products_targets(self, \*\*kwargs) -> ApiResponse:

        Creates one or more targeting expressions.

        body: | REQUIRED {'description': 'An array of asins objects.}'

            | '**campaignId**': *number*, {'description': 'The identifier of the campaign to which this negative target is associated.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which this negative target is associated.'}
            | '**state**': *number*, {'description': 'The current resource state. [ enabled, paused, archived ]'}
            | '**expression**'
            |       '**value**': *string*, {'description': 'The expression value. '}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**expressionType**': *string*, {'description': '[ auto, manual ]'}

        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets', method='PUT')
    def edit_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        edit_negative_targets(self, \*\*kwargs) -> ApiResponse:

        Updates one or more negative targeting clauses.

        body: | REQUIRED {'description': 'An array of asins objects.}'

            | '**campaignId**': *number*, {'description': 'The identifier of the campaign to which this negative target is associated.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which this negative target is associated.'}
            | '**state**': *number*, {'description': 'The current resource state. [ enabled, paused, archived ]'}
            | '**expression**'
            |       '**value**': *string*, {'description': 'The expression value. '}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**expressionType**': *string*, {'description': '[ auto, manual ]'}

        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets', method='GET')
    def list_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        list_negative_targets(self, \*\*kwargs) -> ApiResponse

        Gets a list of negative targeting clauses filtered by specified criteria.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **targetIdFilter**:*string* | Optional. A comma-delimited list of target identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets/{}', method='GET')
    def get_negative_target(self, targetId, **kwargs) -> ApiResponse:
        r"""

        get_negative_targets(self, targetId, \*\*kwargs) -> ApiResponse

        Get a negative targeting clause specified by identifier.

            path **targetId**:*number* | Required. The target identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets/{}', method='DELETE')
    def delete_negative_targets(self, targetId, **kwargs) -> ApiResponse:
        r"""

        delete_negative_targets(self, targetId, \*\*kwargs) -> ApiResponse

        Archives a negative targeting clause.

            path **targetId**:*number* | Required. The target identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets/extended', method='GET')
    def list_negative_targets_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_negative_targets_extended(self, \*\*kwargs) -> ApiResponse

        Gets a list of negative targeting clauses filtered by specified criteria.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **targetIdFilter**:*string* | Optional. A comma-delimited list of target identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeTargets/extended/{}', method='GET')
    def get_negative_target_extended(self, targetId, **kwargs) -> ApiResponse:
        r"""

        get_negative_target_extended(self, targetId, \*\*kwargs) -> ApiResponse

        Get a negative targeting clause specified by identifier.

            path **targetId**:*number* | Required. The target identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)


from .campaigns import Campaigns
from .ad_groups import AdGroups
from .product_ads import ProductAds
from .bid_recommendations import BidRecommendations
from .keywords import Keywords
from .negative_keywords import NegativeKeywords
from .campaign_negative_keywords import CampaignNegativeKeywords
from .suggested_keywords import SuggestedKeywords
from .product_targeting import Targets
from .negative_product_targeting import NegativeTargets
from .reports import Reports
from .snapshots import Snapshots
__all__ = [
    "Campaigns",
    "AdGroups",
    "ProductAds",
    "BidRecommendations",
    "Keywords",
    "NegativeKeywords",
    "CampaignNegativeKeywords",
    "SuggestedKeywords",
    "Targets",
    "NegativeTargets",
    "Reports",
    "Snapshots"
]



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class AdGroups(Client):

    @sp_endpoint('/v2/sp/adGroups', method='POST')
    def create_ad_groups(self, **kwargs) -> ApiResponse:
        r"""
        create_ad_groups(self, \*\*kwargs) -> ApiResponse

        Creates one or more ad groups.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**name**': *string*, {'description': 'A name for the ad group'}
            | '**campaignId**': *number*, {'description': 'An existing campaign to which the ad group is associated'}
            | '**defaultBid**': *number($float)*, {'description': 'A bid value for use when no bid is specified for keywords in the ad group', 'minimum': '0.02'}
            | '**state**': *string*, {'description': 'A name for the ad group', 'Enum': '[ enabled, paused, archived ]'}


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/adGroups', method='PUT')
    def edit_ad_groups(self, **kwargs) -> ApiResponse:
        r"""
        edit_ad_group(self, \*\*kwargs) -> ApiResponse

        Updates one or more ad groups.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group.'}
            | '**name**': *string*, {'description': 'The name of the ad group.'}
            | '**defaultBid**': *number($float)*, {'description': 'The bid value used when no bid is specified for keywords in the ad group.', 'minimum': '0.02'}
            | '**state**': *string*, {'description': 'The current resource state', 'Enum': '[ enabled, paused, archived ]'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/adGroups', method='GET')
    def list_ad_groups(self, **kwargs) -> ApiResponse:
        r"""
        list_ad_groups(self, \*\*kwargs) -> ApiResponse

        Gets an array of AdGroup objects for a requested set of Sponsored Display ad groups. Note that the AdGroup object is designed for performance, and includes a small set of commonly used fields to reduce size. If the extended set of fields is required, use the ad group operations that return the AdGroupResponseEx object.

            query **startIndex**:*integer* | Optional. Sets a cursor into the requested set of campaigns. Use in conjunction with the count parameter to control pagination of the returned array. 0-indexed record offset for the result set, defaults to 0.

            query **count**:*integer* | Optional. Sets the number of AdGroup objects in the returned array. Use in conjunction with the startIndex parameter to control pagination. For example, to return the first ten ad groups set startIndex=0 and count=10. To return the next ten ad groups, set startIndex=10 and count=10, and so on. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived

            query **campaignIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups associated with the campaign identifiers in the specified comma-delimited list.

            query **adGroupIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups with an identifier specified in the comma-delimited list.

            query **name**:*string* | Optional. The returned array includes only ad groups with the specified name.



        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/adGroups/{}', method='GET')
    def get_ad_group(self, adGroupId, **kwargs) -> ApiResponse:
        r"""

        get_ad_group(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets an ad group specified by identifier.

            path **adGroupId**:*number* | Required. The identifier of an existing ad group.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)

    @sp_endpoint('/v2/sp/adGroups/{}', method='DELETE')
    def delete_ad_group(self, adGroupId, **kwargs) -> ApiResponse:
        r"""

        delete_ad_group(self, adGroupId, \*\*kwargs) -> ApiResponse

        Sets the ad group status to archived. Archived entities cannot be made active again. See developer notes for more information.

            path **adGroupId**:*number* | Required. The identifier of an existing ad group.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)

    @sp_endpoint('/v2/sp/adGroups/extended', method='GET')
    def list_ad_groups_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_ad_groups_extended(self, \*\*kwargs) -> ApiResponse

        Gets an array of AdGroup objects for a requested set of Sponsored Display ad groups. Note that the AdGroup object is designed for performance, and includes a small set of commonly used fields to reduce size. If the extended set of fields is required, use the ad group operations that return the AdGroupResponseEx object.

            query: **startIndex**:*integer* | Optional. Sets a cursor into the requested set of campaigns. Use in conjunction with the count parameter to control pagination of the returned array. 0-indexed record offset for the result set, defaults to 0.

            query **count**:*integer* | Optional. Sets the number of AdGroup objects in the returned array. Use in conjunction with the startIndex parameter to control pagination. For example, to return the first ten ad groups set startIndex=0 and count=10. To return the next ten ad groups, set startIndex=10 and count=10, and so on. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived

            query **campaignIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups associated with the campaign identifiers in the specified comma-delimited list.

            query **adGroupIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups with an identifier specified in the comma-delimited list.

            query **name**:*string* | Optional. The returned array includes only ad groups with the specified name.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/adGroups/extended/{}', method='GET')
    def get_ad_group_extended(self, adGroupId, **kwargs) -> ApiResponse:
        r"""

        get_ad_group_extended(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets an ad group that has extended data fields.

            path **adGroupId**:*number* | Required. The identifier of an existing ad group.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class PageAsins(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/pageAsins', method='GET')
    def get_page_asins(self, **kwargs) -> ApiResponse:
        """

        Gets ASIN information for a specified address.

        Keyword Args
            | query **pageUrl**:*string* | Required. For sellers, the address of a Store page. Vendors may also specify the address of a custom landing page. For more information, see the [Stores section](https://advertising.amazon.com/help#GPRM3ZHEXEY5RBFZ) of the Amazon Advertising support center.

        Returns
            asinList
        """
        return self._request(kwargs.pop('path'),  params=kwargs)



import json
import os
import requests
import hashlib
import logging
from cachetools import TTLCache
from ad_api.base import BaseClient

from .credentials import Credentials
from .access_token_response import AccessTokenResponse
from .exceptions import AuthorizationError

cache = TTLCache(maxsize=10, ttl=3600)
grantless_cache = TTLCache(maxsize=10, ttl=3600)

logger = logging.getLogger(__name__)


class AccessTokenClient(BaseClient):
    host = 'api.amazon.com'
    grant_type = 'refresh_token'
    path = '/auth/o2/token'

    def __init__(self, refresh_token=None, account='default', credentials=None):
        super().__init__(account, credentials)
        self.cred = Credentials(refresh_token, self.credentials)

    def _request(self, url, data, headers):
        response = requests.post(url, data=data, headers=headers)
        response_data = response.json()
        if response.status_code != 200:
            error_message = response_data.get('error_description')
            error_code = response_data.get('error')
            raise AuthorizationError(error_code, error_message, response.status_code)
        return response_data

    def get_auth(self) -> AccessTokenResponse:
        """
        Get's the access token
        :return:AccessTokenResponse
        """
        global cache
        cache_key = self._get_cache_key()

        try:
            access_token = cache[cache_key]
        except KeyError:
            cache_ttl = 3600
            access_token = None

            if not access_token:
                request_url = self.scheme + self.host + self.path
                access_token = self._request(request_url, self.data, self.headers)
            else:
                cache_ttl = access_token.get('expires_in')
            cache = TTLCache(maxsize=10, ttl=cache_ttl - 15)
            cache[cache_key] = access_token
        return AccessTokenResponse(**access_token)


    def authorize_auth_code(self, auth_code):
        request_url = self.scheme + self.host + self.path
        res = self._request(
            request_url,
            data=self._auth_code_request_body(auth_code),
            headers=self.headers
        )
        return res

    def _auth_code_request_body(self, auth_code):
        return {
            'grant_type': 'authorization_code',
            'code': auth_code,
            'client_id': self.cred.client_id,
            'client_secret': self.cred.client_secret
        }

    @property
    def data(self):
        return {
            'grant_type': self.grant_type,
            'client_id': self.cred.client_id,
            'refresh_token': self.cred.refresh_token,
            'client_secret': self.cred.client_secret
        }

    @property
    def headers(self):
        return {
            'User-Agent': self.user_agent,
            'content-type': self.content_type
        }

    def _get_cache_key(self, token_flavor=''):
        return 'access_token_' + hashlib.md5(
            (token_flavor + self.cred.refresh_token).encode('utf-8')
        ).hexdigest()



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Reports(Client):
    """
    Use the Amazon Advertising API for Sponsored Products for campaign, ad group, keyword, negative keyword, and product ad management operations. For more information about Sponsored Products, see the Sponsored Products Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/v2/sp/{}/report', method='POST')
    def post_report(self, recordType, **kwargs) -> ApiResponse:
        r"""
        Requests a Sponsored Products report.

        Request the creation of a performance report for all entities of a single type which have performance data to report. Record types can be one of campaigns, adGroups, keywords, productAds, asins, and targets. Note that for asin reports, the report currently can not include metrics associated with both keywords and targets. If the targetingId value is set in the request, the report filters on targets and does not return sales associated with keywords. If the targetingId value is not set in the request, the report filters on keywords and does not return sales associated with targets. Therefore, the default behavior filters the report on keywords. Also note that if both keywordId and targetingId values are passed, the report filters on targets only and does not return keywords.

        Keyword Args
            | path **recordType** (integer): The type of entity for which the report should be generated. Available values : campaigns, adGroups, keywords, productAds, asins, targets [required]

        Request body
            | **stateFilter** (string): [optional] Filters the response to include reports with state set to one of the values in the comma-delimited list. Note that this filter is only valid for reports of the following type and segment. Asins and targets report types are not supported. Enum [ enabled, paused, archived ].
            | **campaignType** (list > string): [required] Enum: The type of campaign. Only required for asins report - don't use with other report types. [sponsoredProducts]
            | **segment** (string) Dimension on which the report is segmented. Note that Search-terms report for auto-targeted campaigns created before 11/14/2018 can be accessed from the /v2/sp/keywords/report resource. Search-terms report for auto-targeted campaigns generated on-and-after 11/14/2018 can be accessed from the /v2/sp/targets/report resource. Also, keyword search terms reports only return search terms that have generated at least one click or one sale. Enum [ query, placement ].
            | **reportDate** (string): [optional] The date for which to retrieve the performance report in YYYYMMDD format. The time zone is specified by the profile used to request the report. If this date is today, then the performance report may contain partial information. Reports are not available for data older than 60 days. For details on data latency, see the Service Guarantees in the developer notes section.
            | **metrics** (string) [optional] A comma-separated list of the metrics to be included in the report. The following tables summarize report metrics which can be requested via the reports interface. Different report types can use different metrics. Note that ASIN reports only return data for either keywords or targets, but not both.

        Returns:
            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), recordType), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/reports/{}', method='GET')
    def get_report(self, reportId, **kwargs) -> ApiResponse:
        r"""
        Gets a previously requested report specified by identifier.

        Keyword Args
            | path **reportId** (number): The report identifier. [required]

        Returns:
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), reportId), params=kwargs)

    def download_report(self, **kwargs) -> ApiResponse:
        r"""
        Downloads the report previously get report specified by location (this is not part of the official Amazon Advertising API, is a helper method to download the report). Take in mind that a direct download of location returned in get_report will return 401 - Unauthorized.

        kwarg parameter **file** if not provided will take the default amazon name from path download (add a path with slash / if you want a specific folder, do not add extension as the return will provide the right extension based on format choosed if needed)

        kwarg parameter **format** if not provided a format will return a url to download the report (this url has a expiration time)

        Keyword Args
            | **url** (string): The location obatined from get_report [required]
            | **file** (string): The path to save the file if mode is download json, zip or gzip. [optional]
            | **format** (string): The mode to download the report: data (list), raw, url, json, zip, gzip. Default (url) [optional]

        Returns:
            ApiResponse
        """
        return self._download(self, params=kwargs)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Eligibility(Client):

    @sp_endpoint('/eligibility/product/list', method='POST')
    def get_eligibility(self, **kwargs) -> ApiResponse:
        r"""

        get_eligibility(self, **kwargs) -> ApiResponse

        Gets advertising eligibility status for a list of products.

        body: | REQUIRED

            '**adType**': *string*, {'description': 'Set to 'sp' to check product eligibility for Sponsored Products advertisements. Set to 'sb' to check product eligibility for Sponsored Brands advertisements. default: sp. [ sp, sb ]'}

            '**productDetailsList**': *dict*, {'asin*': 'An Amazon product identifier.', 'sku': 'A seller product identifier'}

            '**locale**': *string*, {'description': 'Set to the locale string in the table below to specify the language in which the response is returned.'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class AdGroups(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/adGroups', method='GET')
    def list_ad_groups(self, **kwargs) -> ApiResponse:
        r"""

        Gets an array of ad groups associated with the client identifier passed in the authorization header, filtered by specified criteria.

        Keyword Args
            | query **startIndex**:*integer* | Optional. Sets a cursor into the requested set of campaigns. Use in conjunction with the count parameter to control pagination of the returned array. 0-indexed record offset for the result set, defaults to 0.

            | query **count**:*integer* | Optional. Sets the number of AdGroup objects in the returned array. Use in conjunction with the startIndex parameter to control pagination. For example, to return the first ten ad groups set startIndex=0 and count=10. To return the next ten ad groups, set startIndex=10 and count=10, and so on. Defaults to max page size.

            | query **name**:*string* | Optional. The returned array includes only ad groups with the specified name.

            | query **adGroupIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups with an identifier specified in the comma-delimited list.

            | query **campaignIdFilter**:*string* | Optional. The returned array is filtered to include only ad groups associated with the campaign identifiers in the specified comma-delimited list.

            | query **creativeType**:*string* | Optional. Filter by the type of creative the campaign is associated with. To get ad groups associated with non-video campaigns specify 'productCollection'. To get ad groups associated with video campaigns, this must be set to 'video'. Returns all ad groups if not specified. Available values : productCollection, video

        Returns:
            | ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/sb/adGroups/{}', method='GET')
    def get_ad_group(self, adGroupId, **kwargs) -> ApiResponse:
        r"""

        get_ad_group(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets an ad group specified by identifier.

        Keyword Args
            | path **adGroupId**:*number* | Required. The identifier of an existing ad group.

        Returns:
            | ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)



import pprint

class ApiResponse:
    def __init__(self, payload=None, nextToken=None, **kwargs):
        self.payload = payload or kwargs
        self.headers = kwargs
        self.next_token = self.set_next_token(nextToken)

    def __str__(self):
        return pprint.pformat(self.__dict__)

    def set_next_token(self, nextToken=None):
        if nextToken:
            return nextToken
        try:
            return self.payload.get('NextToken', None)
        except AttributeError:
            return None

    @staticmethod
    def set_rate_limit(headers: dict = None):
        try:
            return headers['x-amzn-RateLimit-Limit']
        except (AttributeError, KeyError, TypeError):
            return None



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Recommendations(Client):

    @sp_endpoint('/sb​/recommendations​/creative​/headline', method='POST')
    def list_recommendations(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Brands(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/brands', method='GET')
    def list_brands(self, **kwargs) -> ApiResponse:
        """
        Gets an array of Brand data objects for the Brand associated with the profile ID passed in the header. For more information about Brands, see [Brand Services](https://brandservices.amazon.com/).

        Keyword Args
            | None

        Returns:
            ApiResponse payload:

            | '**brandId**': *string*, {'description': 'The Brand identifier.'}
            | '**brandEntityId**': *string*, {'description': 'The Brand entity identifier.'}
            | '**brandRegistryName**': *string*, {'description': 'The Brand name.'}

        """
        return self._request(kwargs.pop('path'),  params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class NegativeKeywords(Client):

    @sp_endpoint('/v2/sp/negativeKeywords/{}', method='GET')
    def get_negative_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        get_negative_keyword(self, keywordId, \*\*kwargs) -> ApiResponse

        Gets a campaign negative keyword specified by identifier.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords/{}', method='DELETE')
    def delete_negative_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        delete_negative_keyword(self, keywordId, \*\*kwargs) -> ApiResponse

        Archives a campaign negative keyword.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords/extended/{}', method='GET')
    def get_negative_keyword_extended(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        get_negative_keyword_extended(self, keywordId, \*\*kwargs) -> ApiResponse

        Gets a campaign negative keyword that has extended data fields.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords/extended', method='GET')
    def list_negative_keywords_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_negative_keywords_extended(self, \*\*kwargs) -> ApiResponse

        Gets a list of negative keywords that have extended data fields.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list. Available values : negativePhrase, negativeExact.

            query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values :  enabled, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list.

            Returns:

            ApiResponse

            """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords', method='GET')
    def list_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""
        list_negative_keywords(self, \*\*kwargs) -> ApiResponse

        Gets a list of negative keyword objects.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list. Available values : negativePhrase, negativeExact.

            query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values :  enabled, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list..

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords', method='POST')
    def create_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""
        create_negative_keywords(self, \*\*kwargs) -> ApiResponse:

        Creates one or more campaign negative keywords.

        body: | REQUIRED {'description': 'An array of keyword objects.}'

            | '**campaignId**': *number*, {'description': 'The identifer of the campaign to which the keyword is associated.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which this keyword is associated.'}
            | '**state**': *string*, {'description': 'The current resource state.' , 'Enum': '[ enabled ]'}
            | '**keywordText**': *string*, {'description': 'The text of the expression to match against a search query.'}
            | '**matchType**': *string*, {'description': 'The type of match.' , 'Enum': '[ negativeExact, negativePhrase ]'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/negativeKeywords', method='PUT')
    def edit_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""
        edit_negative_keywords(self, \*\*kwargs) -> ApiResponse:

        Updates one or more campaign negative keywords.

        body: | REQUIRED {'description': 'An array of campaign negative keywords with updated values.'}

            | '**keywordId**': *number*, {'description': 'The identifer of the campaign to which the keyword is associated.'}
            | '**state**': *string*, {'description': 'The current resource state.' , 'Enum': '[ enabled, paused, archived ]'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class ProductAds(Client):
    @sp_endpoint('/v2/sp/productAds', method='GET')
    def list_product_ads(self, **kwargs) -> ApiResponse:
        r"""
        list_product_ads(self, \*\*kwargs) -> ApiResponse

        Gets an array of campaigns.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **adIdFilter**:*string* | Optional. Restricts results to product ads associated with the product ad identifiers specified in the comma-delimited list.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/productAds/extended', method='GET')
    def list_product_ads_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_product_ads_extended_request(self, \*\*kwargs) -> ApiResponse

        Gets extended data for a list of product ads filtered by specified criteria.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **adIdFilter**:*string* | Optional. Restricts results to product ads associated with the product ad identifiers specified in the comma-delimited list.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/productAds/{}', method='GET')
    def get_product_ad(self, adId, **kwargs) -> ApiResponse:
        r"""

        get_product_ad_request(self, adId, \*\*kwargs) -> ApiResponse

        Gets a product ad specified by identifier.

            path **adId**:*number* | Required. A product ad identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adId), params=kwargs)

    @sp_endpoint('/v2/sp/productAds/extended/{}', method='GET')
    def get_product_ad_extended(self, adId, **kwargs) -> ApiResponse:
        r"""

        get_product_ad_extended(self, adId, **kwargs) -> ApiResponse

        Gets extended data for a product ad specified by identifier.

            path **adId**:*number* | Required. A product ad identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adId), params=kwargs)

    @sp_endpoint('/v2/sp/productAds/{}', method='DELETE')
    def delete_product_ad(self, adId, **kwargs) -> ApiResponse:
        r"""

        delete_product_ad(self, adId, \*\*kwargs) -> ApiResponse

        Sets the state of a specified product ad to archived. Note that once the state is set to archived it cannot be changed.

            path **adId**:*number* | Required. A product ad identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adId), params=kwargs)

    @sp_endpoint('/v2/sp/productAds', method='PUT')
    def edit_product_ads(self, **kwargs) -> ApiResponse:
        r"""
        edit_product_ads(self, \*\*kwargs) -> ApiResponse

        Updates one or more product ads specified by identifier.

        body: | REQUIRED {'description': 'A list of product ad objects with updated values for the state field.}'

            | '**adId**': *number*, {'description': 'The identifier of an existing campaign to update.'}
            | '**state**': *string*, {'description': 'The current resource state.', 'Enum': '[ enabled, paused, archived ]'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/productAds', method='POST')
    def create_product_ads(self, **kwargs) -> ApiResponse:
        r"""
        create_product_ads(self, \*\*kwargs) -> ApiResponse

        Creates one or more product ads.

        body: | REQUIRED {'description': 'A list of product ads for creation. Note that the SKU field is used by sellers and the ASIN field is used by vendors.'}

            | '**campaignId**': *number*, {'description': 'The campaign identifier.'}
            | '**adGroupId**': *number*, {'description': 'The ad group identifier.'}
            | '**sku**': *string*, {'description': 'The SKU associated with the product. Defined for seller accounts only.'}
            | '**asin**': *string*, {'description': 'The ASIN associated with the product. Defined for vendors only.'}
            | '**state**': *string*, {'description': 'The current resource state.', 'Enum': '[ enabled, paused, archived ]'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Targets(Client):

    @sp_endpoint('/v2/sp/targets', method='POST')
    def create_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        create_products_targets(self, \*\*kwargs) -> ApiResponse:

        Creates one or more targeting expressions.

        body: | REQUIRED {'description': 'An array of asins objects.}'

            | '**campaignId**': *number*, {'description': 'The number or recommendations returned in a single page.'}
            | '**adGroupId**': *number*, {'description': 'The page number in the result set to return.'}
            | '**expression**'
            |       '**value**': *string*, {'description': 'The expression value.'}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**resolvedExpression**'
            |       '**value**': *string*, {'description': 'The expression value.'}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**expressionType**': *string*, {'description': '[ auto, manual ]'}
            | '**bid**': *number*, {'description': 'The bid for ads sourced using the target. Min / Max 0.02 / 1000'}

        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/targets', method='PUT')
    def edit_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        edit_products_targets(self, \*\*kwargs) -> ApiResponse:

        Updates one or more targeting clauses.

        body: | REQUIRED {'description': 'An array of asins objects.}'

            | '**targetId**': *number*, {'description': 'The target id.'}
            | '**state**': *string*, {'description': '[ enabled, paused, archived ]'}
            | '**expression**'
            |       '**value**': *string*, {'description': 'The expression value.'}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**resolvedExpression**'
            |       '**value**': *string*, {'description': 'The expression value.'}
            |       '**type**': *string*, {'description': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | '**expressionType**': *string*, {'description': '[ auto, manual ]'}
            | '**bid**': *number*, {'description': 'The bid for ads sourced using the target. Min / Max 0.02 / 1000'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/targets', method='GET')
    def list_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        list_products_targets(self, \*\*kwargs) -> ApiResponse

        Gets a list of targeting clauses filtered by specified criteria.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **targetIdFilter**:*string* | Optional. A comma-delimited list of target identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/targets/{}', method='GET')
    def get_products_target(self, targetId, **kwargs) -> ApiResponse:
        r"""

        get_products_targets(self, targetId, \*\*kwargs) -> ApiResponse

        Get a targeting clause specified by identifier.

        :param targetId | Required. The target identifier.
        :type targetId path :*number*

        :returns:
            :200: Success. Get a targeting ApiResponse
            :401: Unauthorized

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/v2/sp/targets/{}', method='DELETE')
    def delete_products_target(self, targetId, **kwargs) -> ApiResponse:
        r"""

        delete_products_targets(self, targetId, \*\*kwargs) -> ApiResponse

        Archives a targeting clause.

            path **targetId**:*number* | Required. The target identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/v2/sp/targets/extended', method='GET')
    def list_products_targets_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_products_targets_extended(self, \*\*kwargs) -> ApiResponse

        Gets a list of targeting clauses filtered by specified criteria.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **targetIdFilter**:*string* | Optional. A comma-delimited list of target identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/targets/extended/{}', method='GET')
    def get_products_target_extended(self, targetId, **kwargs) -> ApiResponse:
        r"""

        get_products_targets_extended(self, targetId, \*\*kwargs) -> ApiResponse

        Get a targeting clause specified by identifier.

            path **targetId**:*number* | Required. The target identifier.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/v2/sp/targets/productRecommendations', method='POST')
    def get_products_targets_recommendations(self, **kwargs) -> ApiResponse:
        r"""
        get_products_targets_recommendations(self, \*\*kwargs) -> ApiResponse:

        Gets a list of recommended products for targeting.

            path **keyword**:*string* | Optional Unique exclude categoryId. A keyword for which to get recommended brands.
            path **categoryId**:*number* | Optional Unique exclude keyword. Gets the top 50 brands for the specified category identifier.


        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/targets/brands', method='GET')
    def get_brand_targets(self, **kwargs) -> ApiResponse:
        r"""
        get_brand_targets(self, \*\*kwargs) -> ApiResponse:

        Gets a list of recommended products for targeting.

        body: | REQUIRED {'description': 'An array of asins objects.}'

            | '**pageSize**': *number*, {'description': 'The number or recommendations returned in a single page.'}
            | '**pageNumber**': *number*, {'description': 'The page number in the result set to return.'}
            | '**asins**': *string*, {'description': 'A list of ASINs.'}

        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class History(Client):
    """History of entity changes.
    Provides information about changes made to campaigns, adgroups, ads, etc

    """
    @sp_endpoint('/history', method='POST')
    def get_history(self, **kwargs) -> ApiResponse:
        r"""

        Returns history of changes for provided event sources that match the filters and time ranges specified. Only events that belong to the authenticated Advertiser can be queried. All times will be in UTC Epoch format. This API accepts identifiers in either the alphamumeric format (default), or the numeric format. If numeric IDs are supplied, then numeric IDs will be returned otherwise, alphanumeric IDs are returned.

        body: | REQUIRED {'description': 'A HistoryQuery}'

            | **from_date** | **int** | Max 90 days of history.
            | **to_date** | **int** |
            | **event_types**
                | HistoryEventType
            | **next_token** | **str** | token from previous response to get next set of data. | [optional]
            | **page_offset** | **int** | Mutually exclusive with &#39;nextToken&#39;. Max results with pageOffset is 10000. Use nextToken instead for more results. | [optional]
            | **count** | **int** | Requested number of results. Default 100. Minimum 50. Maximum 200. | [optional]
            | **sort**
                | HistorySortParameter
            | **any string name** | **bool, date, datetime, dict, float, int, list, str, none_type** | any string name can be used but the value must be the correct type | [optional]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Insights(Client):
    """
    Insights API
    """
    @sp_endpoint('/insights/audiences/{}/overlappingAudiences', method='GET')
    def get_insights(self, audienceId, **kwargs) -> ApiResponse:
        r"""
        get_insights(self, audienceId, **kwargs) -> ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), audienceId), params=kwargs)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Campaigns(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/campaigns', method='GET')
    def list_campaigns(self, **kwargs) -> ApiResponse:
        """
        Gets an array of all campaigns associated with the client identifier passed in the authorization header, filtered by specified criteria.

        **Returns both productCollection and video campaigns. Use either 'adFormatFilter' or 'creativeType' to filter campaigns by ad formats (productCollection, video)**.

        Keyword Args
            | **start_index** (int): Sets a zero-based offset into the requested set of campaigns. Use in conjunction with the `count` parameter to control pagination of the returned array.. [optional] if omitted the server will use the default value of 0. Default value : 0
            | **count** (int): Sets the number of campaigns in the returned array. Use in conjunction with the `startIndex` parameter to control pagination. For example, to return the first ten campaigns set `startIndex=0` and `count=10`. To return the next ten campaigns, set `startIndex=10` and `count=10`, and so on. Default value : max page size[optional]
            | **state_filter** (State): The returned array is filtered to include only campaigns with state set to one of the values in the specified comma-delimited list. Defaults to `enabled` and `paused`.
                Note that Campaigns rejected during moderation have state set to `archived`. Available values : enabled, paused, archived[optional]
            | **name** (str): The returned array includes only campaigns with the specified name.. [optional]
            | **portfolio_id_filter** (str): The returned array includes only campaigns associated with Portfolio identifiers matching those specified in the comma-delimited string.. [optional]
            | **campaign_id_filter** (str): The returned array includes only campaigns with identifiers matching those specified in the comma-delimited string.. [optional]
            | **ad_format_filter** (AdFormat): The returned array includes only campaigns with ad format matching those specified in the comma-delimited adFormats. Returns all campaigns if not specified. Available values : productCollection, video[optional]
            | **creative_type** (CreativeType): Filter by the type of creative the campaign is associated with. To get non-video campaigns specify 'productCollection'. To get video campaigns, this must be set to 'video'. Returns all campaigns if not specified. Available values : productCollection, video[optional]

        :meta public:
        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/sb/campaigns', method='POST')
    def create_campaigns(self, **kwargs) -> ApiResponse:
        """
        Creates one or more new Campaigns.

        Request body
            | **name** (string): [optional] The name of the campaign. This name must be unique to the Amazon Advertising account to which the campaign is associated. Maximum length of the string is 128 characters.
            | **tags** (CampaignTags > string): [optional] A list of advertiser-specified custom identifiers for the campaign. Each customer identifier is a key-value pair. You can specify a maximum of 50 identifiers.
            | **budget** (float): [optional] The budget amount associated with the campaign.
            | **budget_type** (BudgetType > string) [optional] Note that for the lifetime budget type, `startDate` and `endDate` must be specified. The lifetime budget range is from 100 to 20,000,000 and daily budget range is 1 to 1,000,000 by default for most marketplaces. For the JP marketplace, the lifetime budget range is fromt 10,000 to 2,000,000,000, and the daily budget range is 100 to 21,000,000.., must be one of ["lifetime", "daily", ]
            | **start_date** (StartDate > string) [optional] The YYYYMMDD start date of the campaign. Must be equal to or greater than the current date. If this property is not included in the request, the startDate value is not updated. If set to null, startDate is set to the current date. [nullable: true] [pattern: ^\d{8}$]
            | **end_date** (EndDate > string) [optional] The YYYYMMDD end date of the campaign. Must be greater than the value specified in the startDate field. If this property is not included in the request, the endDate value is not updated. If set to null, endDate is deleted from the draft campaign. [nullable: true] [pattern: ^\d{8}$]
            | **ad_format** (AdFormat > string) [optional] The type of ad format. Enum: [ productCollection, video ]
            | **state** (State > string): [optional] Enum: [ enabled, paused, archived ]
            | **brand_entity_id** (str, writeOnly: true) [optional] The brand entity identifier. Note that this field is required for sellers. For more information, see the [Stores reference](https://advertising.amazon.com/API/docs/v2/reference/stores) or [Brands reference](https://advertising.amazon.com/API/docs/v3/reference/SponsoredBrands/Brands).
            | **bid_optimization** (bool) [optional] Set to `true` to allow Amazon to automatically optimize bids for placements below top of search if omitted the server will use the default value of True
            | **bid_multiplier** (float minimum: -99 maximum: 99) [optional] A bid multiplier. Note that this field can only be set when 'bidOptimization' is set to false. Value is a percentage to two decimal places. Example: If set to -40.00 for a $5.00 bid, the resulting bid is $3.00.
            | **portfolio_id** (int) [optional] The identifier of the portfolio to which the campaign is associated.
            | **creative** (SBCreative): [optional]
                | **brand_name** (str) A brand name. Maximum length is 30 characters.

                | **brand_logo_asset_id** (str) The identifier of the brand logo image from the Store assets library. See [listAssets](https://advertising.amazon.com/API/docs/v3/reference/SponsoredBrands/Stores) for more information. Note that for campaigns created in the Amazon Advertising console prior to release of the Store assets library, responses will not include a value for the brandLogoAssetID field.
                | **brand_logo_url** (str readOnly: true) The address of the hosted image.
                | **headline** (str) The headline text. Maximum length of the string is 50 characters for all marketplaces other than Japan, which has a maximum length of 35 characters.. [optional]
                | **asins** (str) An array of ASINs associated with the creative.

                    **NOTE**: do not pass an empty array, this results in an error.

                | **should_optimize_asins** (bool) default: false

                    **NOTE**: Starting on March 25th, 2021, this property will no longer be supported. This feature is currently available in the US and UK. Existing Sponsored Brands campaigns with product optimization enabled will no longer have the products in the creative automatically optimized. Campaigns with product optimization enabled will be converted to standard Sponsored Brands product collection campaigns with the default selected products showing in the creative. For POST and PUT operations, setting this property to `true` will not have any effect. The value returned in the response will always be `false`. For the GET operation, the value of this field will always be `false`. And starting on September 25th, 2021, this property will be removed completely. . [optional] if omitted the server will use the default value of False

            | **landing_page** (SBLandingPage): [optional]

                **asins** ([str]) An array of ASINs used to generate a simple landing page. The response includes the URL of the generated simple landing page. Do not include this property in the request if the `url` property is also included, these properties are mutually exclusive.

                **url** (str) URL of an existing simple landing page or Store page. Vendors may also specify the URL of a custom landing page. If a custom URL is specified, the landing page must include the ASINs of at least three products that are advertised as part of the campaign. Do not include this property in the request if the `asins` property is also included, these properties are mutually exclusive.

            | **keywords** ([SBCommonKeywordsKeywords]): An array of keywords associated with the campaign. [optional]

                **keyword_text** (str) The keyword text. Maximum of 10 words.. [optional]

                **native_language_keyword** (str) The unlocalized keyword text in the preferred locale of the advertiser.. [optional]

                **native_language_locale** (str) The locale preference of the advertiser. For example, if the advertiser’s preferred language is Simplified Chinese, set the locale to `zh_CN`. Supported locales include: Simplified Chinese (locale: zh_CN) for US, UK and CA. English (locale: en_GB) for DE, FR, IT and ES.. [optional]

                **match_type** (MatchType) [optional] The match type. For more information, see match types in the Amazon Advertising support center. Enum: [ broad, exact, phrase ]

                **bid** (float) The associated bid. Note that this value must be less than the budget associated with the Advertiser account. For more information, see [supported features](https://advertising.amazon.com/API/docs/v2/guides/supported_features) [optional]

            | **negative_keywords** ([SBUpdateDraftCampaignRequestWithKeywordsAllOfNegativeKeywords]): An array of negative keywords associated with the campaign.[optional]

                **keyword_text** (str): The keyword text. Maximum of 10 words. [optional]

                **match_type** (NegativeMatchType): [optional] The negative match type. For more information, see negative keyword match types in the Amazon Advertising support center. Enum:[ negativeExact, negativePhrase ]
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/campaigns', method='PUT')
    def edit_campaigns(self, **kwargs) -> ApiResponse:

        """
        Updates one or more campaigns.

        Mutable fields:
            | name
            | state
            | portfolioId
            | budget
            | bidOptimization
            | bidMultiplier
            | endDate

        Request body
            | **name** (string): [optional] The name of the campaign. This name must be unique to the Amazon Advertising account to which the campaign is associated. Maximum length of the string is 128 characters.
            | **state** (State > string): [optional] Enum: [ enabled, paused, archived ]
            | **portfolio_id** (int) [optional] The identifier of the portfolio to which the campaign is associated.
            | **budget** (float): [optional] The budget amount associated with the campaign.
            | **bid_optimization** (bool) [optional] Set to `true` to allow Amazon to automatically optimize bids for placements below top of search if omitted the server will use the default value of True
            | **bid_multiplier** (float minimum: -99 maximum: 99) [optional] A bid multiplier. Note that this field can only be set when 'bidOptimization' is set to false. Value is a percentage to two decimal places. Example: If set to -40.00 for a $5.00 bid, the resulting bid is $3.00.
            | **end_date** (EndDate > string) [optional] The YYYYMMDD end date of the campaign. Must be greater than the value specified in the startDate field. If this property is not included in the request, the endDate value is not updated. If set to null, endDate is deleted from the draft campaign. [nullable: true] [pattern: ^\d{8}$]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/campaigns/{}', method='GET')
    def get_campaign(self, campaignId, **kwargs) -> ApiResponse:
        """
        Gets a campaign specified by identifier.

        Keyword Args
            | query **campaignId** (integer): The identifier of an existing campaign. [required]
            | query **locale** (string): The returned array includes only keywords associated with locale matching those specified by identifier. [optional]




        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)

    @sp_endpoint('/sb/campaigns/{}', method='DELETE')
    def delete_campaign(self, campaignId, **kwargs) -> ApiResponse:
        """
        Archives a campaign specified by identifier.

        This operation is equivalent to an update operation that sets the status field to 'archived'. Note that setting the status field to 'archived' is permanent and can't be undone. See Developer Notes for more information

        Keyword Args
            | query **campaignId** (integer): The identifier of an existing campaign. [required]

        Returns
            ApiResponse


        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class SuggestedKeywords(Client):

    @sp_endpoint('/v2/sp/adGroups/{}/suggested/keywords', method='GET')
    def get_keywords(self, adGroupId, **kwargs) -> ApiResponse:
        r"""
        get_keywords_request(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets suggested keywords for the specified ad group.

            path adGroupId:*number* | Required. The identifier of a valid ad group.

            query maxNumSuggestions:*integer* | Optional. The maxiumum number of suggested keywords for the response. Default value 100.

            query adStateFilter:*string* | Optional. Filters results to ad groups with state matching the comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)

    @sp_endpoint('/v2/sp/adGroups/{}/suggested/keywords/extended', method='GET')
    def get_keywords_extended(self, adGroupId, **kwargs) -> ApiResponse:
        r"""
        get_keywords_extended(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets suggested keywords with extended data for the specified ad group.

            path adGroupId:*number* | Required. The identifier of a valid ad group.

            query maxNumSuggestions:*integer* | Optional. The maxiumum number of suggested keywords for the response. Default value 100.

            query suggestBids:*string* | Optional. Set to yes to include a suggest bid for the suggested keyword in the response. Otherwise, set to no. Available values : yes, no. Default value : no.

            query adStateFilter:*string* | Optional. Filters results to ad groups with state matching the comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)

    @sp_endpoint('/v2/sp/asins/{}/suggested/keywords', method='GET')
    def get_asin_keywords(self, asinValue, **kwargs) -> ApiResponse:
        r"""
        get_asin_keywords(self, asinValue, \*\*kwargs) -> ApiResponse

        Gets suggested keywords for the specified ad group.

            path asinValue:*string* | Required. An ASIN.

            query maxNumSuggestions:*integer* | Optional. The maxiumum number of suggested keywords for the response. Default value 100.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), asinValue), params=kwargs)

    @sp_endpoint('/v2/sp/asins/suggested/keywords', method='POST')
    def get_asins_keywords(self, **kwargs) -> ApiResponse:
        r"""
        get_asins_keywords(self, \*\*kwargs) -> ApiResponse:

        Gets suggested keyword for a specified list of ASINs.

        body: | REQUIRED {'description': 'An object with ASINs.}'

            | 'asins': *string*, {'description': 'A list of ASINs.'}
            | 'maxNumSuggestions': *integer*, {'description': 'The maximum number of suggested keywords in the response. minItems: 1. maxItems: 1000. default: 100'}


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



import json
import os
import confuse
from cachetools import Cache
import sys

class MissingCredentials(Exception):
    """
    Credentials are missing, see the error output to find possible causes
    """
    pass


class CredentialProvider:
    credentials = None
    cache = Cache(maxsize=10)

    def __init__(self, account='default', credentials=None):
        self.account = account
        self.read_credentials = [
            self.read_config
        ]
        if credentials:
            self.credentials = self.Config(**credentials)
            missing = self.credentials.check_config()
            if len(missing):
                raise MissingCredentials(f'The following configuration parameters are missing: {missing}')
        else:
            self.load_credentials()

    def load_credentials(self):
        for read_method in self.read_credentials:
            if read_method():
                return True

    def read_config(self):
        try:
            config = confuse.Configuration('python-ad-api')
            config_filename = os.path.join(config.config_dir(), 'credentials.yml')
            config.set_file(config_filename)
            account_data = config[self.account].get()
            self.credentials = self.Config(**account_data)
            missing = self.credentials.check_config()
            if len(missing):
                raise MissingCredentials(f'The following configuration parameters are missing: {missing}')
        except confuse.exceptions.NotFoundError:
            raise MissingCredentials(f'The account {self.account} was not setup in your configuration file.')
        except confuse.exceptions.ConfigReadError:
            raise MissingCredentials(
                f'Neither environment variables nor a config file were found. '
                f'Please set the correct variables, or use a config file (credentials.yml). '
                f'See https://confuse.readthedocs.io/en/latest/usage.html#search-paths for search paths.'
            )
        else:
            return True

    class Config:
        def __init__(self,
                     refresh_token,
                     client_id,
                     client_secret,
                     profile_id
                     ):
            self.refresh_token = refresh_token
            self.client_id = client_id
            self.client_secret = client_secret
            self.profile_id = profile_id

        def check_config(self):
            errors = []
            for k, v in self.__dict__.items():
                if not v and k != 'refresh_token':
                    errors.append(k)
            return errors



from io import BytesIO

from Crypto.Util.Padding import pad
import hashlib

import base64
from Crypto.Cipher import AES

def fill_query_params(query, *args):
    return query.format(*args)


def sp_endpoint(path, method='GET'):
    def decorator(function):
        def wrapper(*args, **kwargs):
            kwargs.update({
                'path': path,
                'method': method
            })
            return function(*args, **kwargs)

        wrapper.__doc__ = function.__doc__
        return wrapper

    return decorator


def encrypt_aes(file_or_bytes_io, key, iv):
    key = base64.b64decode(key)
    iv = base64.b64decode(iv)
    aes = AES.new(key, AES.MODE_CBC, iv)
    try:
        if isinstance(file_or_bytes_io, BytesIO):
            return aes.encrypt(pad(file_or_bytes_io.read(), 16))
        return aes.encrypt(pad(bytes(file_or_bytes_io.read(), encoding='iso-8859-1'), 16))
    except UnicodeEncodeError:
        file_or_bytes_io.seek(0)
        return aes.encrypt(pad(bytes(file_or_bytes_io.read(), encoding='utf-8'), 16))
    except TypeError:
        file_or_bytes_io.seek(0)
        return aes.encrypt(pad(file_or_bytes_io.read(), 16))


def decrypt_aes(content, key, iv):
    key = base64.b64decode(key)
    iv = base64.b64decode(iv)
    decrypter = AES.new(key, AES.MODE_CBC, iv)
    decrypted = decrypter.decrypt(content)
    padding_bytes = decrypted[-1]
    return decrypted[:-padding_bytes]


def create_md5(file):
    hash_md5 = hashlib.md5()
    if isinstance(file, BytesIO):
        for chunk in iter(lambda: file.read(4096), b''):
            hash_md5.update(chunk)
        file.seek(0)
        return hash_md5.hexdigest()
    if isinstance(file, str):
        with open(file, "rb") as f:
            for chunk in iter(lambda: f.read(4096), b''):
                hash_md5.update(chunk)
        return hash_md5.hexdigest()
    for chunk in iter(lambda: file.read(4096), b''):
        hash_md5.update(chunk)
    return hash_md5.hexdigest()


def nest_dict(flat: dict()):
    result = {}
    for k, v in flat.items():
        _nest_dict_rec(k, v, result)
    return result


def _nest_dict_rec(k, v, out):
    k, *rest = k.split('.', 1)
    if rest:
        _nest_dict_rec(rest[0], v, out.setdefault(k, {}))
    else:
        out[k] = v



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Media(Client):
    """
    Use this interface to request and retrieve store information. This can be used for Sponsored Brands campaign creation, to pull the store URL information, and for asset registration for Stores.
    """
    @sp_endpoint('/media/upload', method='POST')
    def create_media(self, **kwargs) -> ApiResponse:
        """
        Creates an ephemeral resource (upload location) to upload Media for an Ad Program (SponsoredBrands).

        The upload location is short lived and expires in 15 minutes. Once the upload is complete, /media/complete API should be used to notify that the upload is complete.

        The upload location only supports PUT HTTP Method to upload the media content. If the upload location expires, API user will get 403 Forbidden response.

        Request body
            | **programType** (string): [required] [ SponsoredBrands ].
            | **creativeType** (string): [required] [ Video ].

        Returns:
            | ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/media/complete', method='PUT')
    def complete_media(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/media/describe', method='GET')
    def describe_media(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


import os

class Credentials:
    def __init__(self, refresh_token, credentials):
        self.client_id = credentials.client_id
        self.client_secret = credentials.client_secret
        self.refresh_token = refresh_token or credentials.refresh_token
        self.profile_id = credentials.profile_id



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Moderation(Client):
    """
    Use this interface to request and retrieve store information. This can be used for Sponsored Brands campaign creation, to pull the store URL information, and for asset registration for Stores.
    """
    @sp_endpoint('/sb/moderation/campaigns/{}', method='GET')
    def get_moderation(self, campaignId, **kwargs) -> ApiResponse:
        """
        Gets the moderation result for a campaign specified by identifier.

        Keyword Args
            | path **campaignId** (integer): The campaign identifier. [required]

        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)



from ad_api.base.credential_provider import CredentialProvider

class BaseClient:
    scheme = 'https://'
    method = 'GET'
    content_type = 'application/x-www-form-urlencoded;charset=UTF-8'
    user_agent = 'python-ad-api'

    def __init__(self, account='default', credentials=None):
        try:
            import pkg_resources
            version = pkg_resources.require("python-amazon-ad-api")[0].version
            self.user_agent += f'-{version}'
        except:
            pass
        self.credentials = CredentialProvider(account, credentials).credentials



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Targets(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/targets/list', method='POST')
    def list_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        Gets a list of product targets associated with the client identifier passed in the authorization header, filtered by specified criteria.

        Request Body
            | '**nextToken**': *string*, {'description': 'Operations that return paginated results include a pagination token in this field. To retrieve the next page of results, call the same operation and specify this token in the request. If the NextToken field is empty, there are no further results.'}
            | '**maxResults**': *number*, {'description': 'The identifier of the ad group to which this keyword is associated.'}
            | '**filters**': *string*, {'Restricts results to targets with the specified filters. Filters are inclusive. Filters are joined using 'and' logic. Specify one type of each filter. Specifying multiples of the same type of filter results in an error.'}

                | '**filterType**': *string*, {'CREATIVE_TYPE': '[ CREATIVE_TYPE ]'}

                | '**filterType**': *string*, {'TARGETING_STATE': '[ archived, paused, pending, enabled ]'}

                | '**filterType**': *string*, {'CAMPAIGN_ID': '[ CAMPAIGN_ID ]'}

                | '**filterType**': *string*, {'AD_GROUP_ID': '[ SBAdGroupId ]'}


        Returns:
            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


    @sp_endpoint('/sb/targets', method='PUT')
    def edit_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        Updates one or more targets.

        Request Body
            | '**targetId**': *integer($int64)*, {'description': 'The identifier of the target.'}
            | '**adGroupId**': *integer($int64)*, {'description': 'The identifier of the ad group to which the target is associated.'}
            | '**campaignId**': *integer($int64)*, {'description': 'The identifier of the campaign to which the target is associated.'}
            | '**state**': *string*, {'values': '[ enabled, paused, pending, archived, draft ]'}
            | '**bid**': *number*, {'description': 'The associated bid. Note that this value must be less than the budget associated with the Advertiser account. For more information, see supported features.'}

        Returns:
            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/targets', method='POST')
    def create_products_targets(self, **kwargs) -> ApiResponse:
        r"""
        Create one or more targets.

        Request Body
            | '**adGroupId**': *integer($int64)*, {'description': 'The identifier of the ad group to which the target is associated.'}
            | '**campaignId**': *integer($int64)*, {'description': 'The identifier of the campaign to which the target is associated.'}
            | '**expressions**': *SBExpression*, {'type': 'asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs', 'values': 'The text of the targeting expression. The - token defines a range. For example, 2-4 defines a range of 2, 3, and 4.'}
            | '**bid**': *number*, {'description': 'The associated bid. Note that this value must be less than the budget associated with the Advertiser account. For more information, see supported features.'}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/targets/{}', method='GET')
    def get_products_target(self, targetId, **kwargs) -> ApiResponse:
        r"""
        Gets a target specified by identifier.

        Keyword Args
            path **targetId**:*number* | Required. The identifier of an existing target.

        Returns
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)

    @sp_endpoint('/sb/targets/{}', method='DELETE')
    def delete_products_target(self, targetId, **kwargs) -> ApiResponse:
        r"""
        Archives a target specified by identifier. Note that archiving is permanent, and once a target has been archived it can't be made active again.

        Keyword Args
            path **targetId**:*number* | Required. The identifier of an existing target.

        Returns
            ApiResponse


        """
        return self._request(fill_query_params(kwargs.pop('path'), targetId), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Localization(Client):
    """
    This API provides operations to localize data used when creating advertising campaigns. Depending on the type of data, localization may entail translating text, converting monetary amounts, or mapping an entity in a source marketplace to an analogous entity in one or more target marketplaces.
    """
    @sp_endpoint('/currencies/localize', method='POST')
    def get_currency(self, **kwargs) -> ApiResponse:
        r"""

        Gets an array of localized currencies in their target marketplaces, with the advertiser ID and source marketplace ID passed in through the header and body

        Returns localized currencies within specified marketplaces.

        **Requires one of these permissions**: [\"advertiser_campaign_edit\",\"advertiser_campaign_view\"]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/products/localize', method='POST')
    def get_products(self, **kwargs) -> ApiResponse:
        r"""

        Localizes (maps) products from a source marketplace to one or more target marketplaces. The localization process succeeds for a given target marketplace if a product matching the source product can be found there and the advertiser is eligible to advertise it. Seller requests have an additional condition: the SKU of a localized product must match the SKU of the source product.

        **Requires one of these permissions**: [\"advertiser_campaign_edit\",\"advertiser_campaign_view\"]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/keywords/localize', method='POST')
    def get_keywords(self, **kwargs) -> ApiResponse:
        r"""
        Returns localized keywords.

        Returns localized keywords within specified marketplaces or locales.

        **Requires one of these permissions**: [\"advertiser_campaign_edit\",\"advertiser_campaign_view\"]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/targetingExpression/localize', method='POST')
    def get_targeting_expression(self, **kwargs) -> ApiResponse:
        r"""
        Localizes targeting expressions used for advertising targeting.

        Localizes (maps) targeting expressions from a source marketplace to one or more target marketplaces.

        **Requires one of these permissions**: [\"advertiser_campaign_edit\",\"advertiser_campaign_view\"]

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)




from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class NegativeTargets(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/negativeTargets/list', method='POST')
    def list_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        Gets a list of product negative targets associated with the client identifier passed in the authorization header, filtered by specified criteria.

        Request Body
            | '**nextToken**': *string*, {'description': 'Operations that return paginated results include a pagination token in this field. To retrieve the next page of results, call the same operation and specify this token in the request. If the NextToken field is empty, there are no further results.'}
            | '**maxResults**': *number*, {'description': 'The identifier of the ad group to which this keyword is associated.'}
            | '**filters**': *string*, {'Restricts results to targets with the specified filters. Filters are inclusive. Filters are joined using 'and' logic. Specify one type of each filter. Specifying multiples of the same type of filter results in an error.'}

                | '**filterType**': *string*, {'CREATIVE_TYPE': '[ productCollection, video ]'}

                | '**filterType**': *string*, {'TARGETING_STATE': '[ enabled, pending, archived ]'}

                | '**filterType**': *string*, {'CAMPAIGN_ID': '[ CAMPAIGN_ID ]'}

                | '**filterType**': *string*, {'AD_GROUP_ID': '[ SBAdGroupId ]'}


        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/negativeTargets', method='PUT')
    def edit_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        Updates one or more negative targeting clauses.

        Request Body
            | '**targetId**': *integer($int64)*, {'description': 'The target identifier.'}
            | '**adGroupId**': *integer($int64)*, {'description': 'The identifier of an existing ad group. The newly created target is associated to this ad group.'}
            | '**state**': *string*, {'values': '[ enabled, paused, pending, archived, draft ]'}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/negativeTargets', method='POST')
    def create_negative_targets(self, **kwargs) -> ApiResponse:
        r"""
        Creates one or more targeting expressions.

        Request Body

            | '**adGroupId**': *integer($int64)*, {'description': 'The identifier of the ad group to which the target is associated.'}
            | '**campaignId**': *integer($int64)*, {'description': 'The identifier of the campaign to which the target is associated.'}
            | '**expressions**': *string*, {'Restricts results to targets with the specified filters. Filters are inclusive. Filters are joined using 'and' logic. Specify one type of each filter. Specifying multiples of the same type of filter results in an error.'}

                | '**type**': *string*, {'values': 'asinBrandSameAs, asinSameAs'}

                | '**value**': *string*, {'description': 'The text of the negative expression.'}

        Returns:
            ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/negativeTargets/{}', method='GET')
    def get_negative_target(self, negativeTargetId, **kwargs) -> ApiResponse:
        r"""
        Get a negative targeting clause specified by identifier.

        Keyword Args
            path **negativeTargetId**:*number* | Required. The identifier of an existing negative target.

        Returns
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), negativeTargetId), params=kwargs)

    @sp_endpoint('/sb/negativeTargets/{}', method='DELETE')
    def delete_negative_target(self, negativeTargetId, **kwargs) -> ApiResponse:
        r"""
        Archives a negative targeting clause.

        Keyword Args
            path **negativeTargetId**:*number* | Required. The identifier of an existing negative target.

        Returns
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), negativeTargetId), params=kwargs)



from .campaigns import Campaigns
from .brands import Brands
from .landing_page_asins import PageAsins
from .ad_groups import AdGroups
from .moderation import Moderation
from .keywords import Keywords
from .recommendations import Recommendations
from .keywords_recommendations import KeywordsRecommendations
from .forecast import Forecast
from .negative_keywords import NegativeKeywords
from .product_targeting import Targets
from .negative_product_targeting import NegativeTargets
from .targeting_recommendations import TargetsRecommendations
from .bid_recommendations import BidRecommendations
from .stores import Stores
from .media import Media
__all__ = [
    "Brands",
    "PageAsins",
    "Campaigns",
    "AdGroups",
    "Moderation",
    "Keywords",
    "Recommendations",
    "KeywordsRecommendations",
    "Forecast",
    "NegativeKeywords",
    "Targets",
    "NegativeTargets",
    "TargetsRecommendations",
    "BidRecommendations",
    "Stores",
    "Media"
]



from .base_client import BaseClient
from .client import Client
from .helpers import fill_query_params, sp_endpoint
from .marketplaces import Marketplaces
from .exceptions import AdvertisingApiException
from .exceptions import AdvertisingApiBadRequestException
from .exceptions import AdvertisingApiForbiddenException
from .credential_provider import CredentialProvider, MissingCredentials
from .api_response import ApiResponse

__all__ = [
    'AccessTokenClient',
    'ApiResponse',
    'Client',
    'BaseClient',
    'Marketplaces',
    'fill_query_params',
    'sp_endpoint',
    'AdvertisingApiException',
    'AdvertisingApiBadRequestException',
    'AdvertisingApiForbiddenException',
    'CredentialProvider',
    'MissingCredentials'
]



import json
from datetime import datetime
import logging
from cachetools import TTLCache
from requests import request
from ad_api.auth import AccessTokenClient, AccessTokenResponse
from .api_response import ApiResponse
from .base_client import BaseClient
from .exceptions import get_exception_for_code, get_exception_for_content, AdvertisingApiBadRequestException
from .marketplaces import Marketplaces
import sys
import os
import requests
from io import BytesIO
import gzip
from zipfile import ZipFile
import zipfile

from urllib.parse import urlparse

log = logging.getLogger(__name__)
role_cache = TTLCache(maxsize=10, ttl=3600)

class Client(BaseClient):
    grantless_scope = ''
    def __init__(
            self,
            account='default',
            marketplace: Marketplaces = Marketplaces.EU,
            refresh_token=None,
            credentials=None
    ):

        super().__init__(account, credentials)
        self.endpoint = marketplace.endpoint
        self._auth = AccessTokenClient(refresh_token=refresh_token, account=account, credentials=credentials)

    @property
    def headers(self):
        return {
            'user-agent': self.user_agent,
            'Amazon-Advertising-API-ClientId': self.credentials.client_id,
            'Authorization': 'Bearer %s' % self.auth.access_token,
            'Amazon-Advertising-API-Scope': self.credentials.profile_id,
            'content-type': 'application/json'
        }

    @property
    def auth(self) -> AccessTokenResponse:
        return self._auth.get_auth()

    @staticmethod
    def _download(self, params: dict = None, headers=None) -> ApiResponse:

        location = params.get("url")

        try:
            r = requests.get(location, headers=self.headers, data=None, allow_redirects=True)

        except requests.exceptions.InvalidSchema as e:
            error = {
                'success': False,
                'code': 400,
                'response': e
            }
            next_token = None
            return ApiResponse(error, next_token, headers=self.headers)
        except requests.exceptions.RequestException as e:
            error = {
                'success': False,
                'code': 503,
                'response': e
            }
            next_token = None
            return ApiResponse(error, next_token, headers=self.headers)
        except requests.exceptions.ConnectionError as e:
            error = {
                'success': False,
                'code': e.status_code,
                'response': e
            }
            next_token = None
            return ApiResponse(error, next_token, headers=self.headers)

        bytes = r.content
        mode = params.get("format")

        if mode is None:
            mode = "url"

        name = params.get("file")

        if name is None:
            o = urlparse(r.url)
            name = o.path[1:o.path.find('.')]


        if mode == "raw":

            next_token = None
            return ApiResponse(bytes, next_token, headers=r.headers)

        elif mode == "url":

            next_token = None
            return ApiResponse(r.url, next_token, headers=r.headers)

        elif mode == "data":

            if bytes[0:2] == b'\x1f\x8b':
                logging.info("Is gzip report")
                buf = BytesIO(bytes)
                f = gzip.GzipFile(fileobj=buf)
                read_data = f.read()
                next_token = None
                return ApiResponse(json.loads(read_data.decode('utf-8')), next_token, headers=r.headers)

            else:
                logging.info("Is bytes snapshot")
                next_token = None
                return ApiResponse(json.loads(r.text), next_token, headers=r.headers)

        elif mode == "json":
            if bytes[0:2] == b'\x1f\x8b':
                buf = BytesIO(bytes)
                f = gzip.GzipFile(fileobj=buf)
                read_data = f.read()
                fo = open(name+".json", 'w')
                fo.write(read_data.decode('utf-8'))
                fo.close()
                next_token = None
                return ApiResponse(name+".json", next_token, headers=r.headers)
            else:
                fo = open(name+".json", 'w')
                fo.write(r.text)
                fo.close()
                next_token = None
                return ApiResponse(name+".json", next_token, headers=r.headers)

        elif mode == "gzip":
            fo = gzip.open(name + ".json.gz", 'wb').write(r.content)
            next_token = None
            return ApiResponse(name + ".json.gz", next_token, headers=r.headers)

        elif mode == "zip":

            if bytes[0:2] == b'\x1f\x8b':
                buf = BytesIO(bytes)
                f = gzip.GzipFile(fileobj=buf)
                read_data = f.read()
                fo = open(name+".json", 'w')
                fo.write(read_data.decode('utf-8'))
                fo.close()

                zipObj = ZipFile(name+'.zip', 'w', zipfile.ZIP_DEFLATED)
                zipObj.write(name + ".json")
                zipObj.close()
            else:
                fo = open(name + ".json", 'w')
                fo.write(r.text)
                fo.close()

                zipObj = ZipFile(name+'.zip', 'w', zipfile.ZIP_DEFLATED)
                zipObj.write(name + ".json")
                zipObj.close()

            if os.path.exists(name + ".json"):
                os.remove(name + ".json")

            next_token = None
            return ApiResponse(name + ".zip", next_token, headers=r.headers)


        else:

            error = {
                'success': False,
                'code': 400,
                'response': 'The mode "%s" is not supported perhaps you could use "data", "raw", "url", "json", "zip" or "gzip"' % (mode)
            }
            next_token = None
            return ApiResponse(error, next_token, headers=self.headers)

        sys.exit()



    def _request(self, path: str, *, data: str = None, params: dict = None, headers=None,
                 add_marketplace=True) -> ApiResponse:
        if params is None:
            params = {}
        if data is None:
            data = {}

        method = params.pop('method')

        if method in ('POST', 'PUT', 'PATCH'):

            if headers is not None:
                headers.update(self.headers)

            res = request(method,
                      self.endpoint + path,
                      params=params,
                      data=data,
                      headers=headers or self.headers)

        else:
            res = request(method,
                      self.endpoint + path,
                      params=params,
                      headers=headers or self.headers)

        return self._check_response(res)

    @staticmethod
    def _check_response(res) -> ApiResponse:

        content = vars(res).get('_content')
        str_content = content.decode('utf8')
        data = json.loads(str_content)

        if type(data) is dict and data.get('code')=='UNAUTHORIZED':
            exception = get_exception_for_content(data)
            raise exception(data)

        if vars(res).get('_content') == b'[]' and vars(res).get('_content_consumed') is True:
            data = json.loads('{"status_code": 200, "msg": "No Data Available"}')

        headers = vars(res).get('headers')
        status_code = vars(res).get('status_code')
        next_token = vars(res).get('_next')
        return ApiResponse(data, next_token, headers=headers)



from .profiles import Profiles
from .invoices import Invoices
from .eligibility import Eligibility
from .metadata import Metadata
from .history import History
from .localization import Localization
from .audiences import Audiences
from .portfolios import Portfolios
from .insights import Insights
__all__ = [
    "Profiles",
    "Invoices",
    "Eligibility",
    "Metadata",
    "History",
    "Localization",
    "Audiences",
    "Portfolios",
    "Insights"
]


class AdvertisingApiException(Exception):
    code = 999
    def __init__(self, error):
        try:
            self.amzn_code = error
        except IndexError:
            pass
        self.error = error

class AdvertisingApiBadRequestException(AdvertisingApiException):
    """
    400	Request has missing or invalid parameters and cannot be parsed.
    """
    code = 400

    def __init__(self, error):
        super(AdvertisingApiBadRequestException, self).__init__(error)


class AdvertisingApiForbiddenException(AdvertisingApiException):
    """
    403	Indicates access to the resource is forbidden. Possible reasons include Access Denied, Unauthorized, Expired Token, or Invalid Signature.
    """
    code = 403

    def __init__(self, error):
        super(AdvertisingApiForbiddenException, self).__init__(error)


def get_exception_for_code(code: int):
    return {
        400: SellingApiBadRequestException,
        403: SellingApiForbiddenException,
        429: SellingApiRequestThrottledException,
        500: SellingApiServerException,
        503: SellingApiTemporarilyUnavailableException
    }.get(code, SellingApiException)


def get_exception_for_content(content: object):
    return {
        'UNAUTHORIZED': AdvertisingApiForbiddenException
    }.get(content.get('code'), AdvertisingApiException)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class NegativeKeywords(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/negativeKeywords', method='GET')
    def list_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""

        Gets an array of negative keywords, filtered by optional criteria.

        Keyword Args
            | query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0
            | query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.
            | query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list. Available values : negativePhrase, negativeExact.
            | query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.
            | query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values :  enabled, archived.
            | query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.
            | query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.
            | query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list.
            | query **creativeType**:*string* | Optional. Filter by the type of creative the campaign is associated with. To get negative keywords associated with non-video campaigns specify 'productCollection'. To get negative keywords associated with video campaigns, this must be set to 'video'. Returns all negative keywords if not specified. Available values : productCollection, video
        Returns:
            | ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)


    @sp_endpoint('/sb/negativeKeywords', method='PUT')
    def edit_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""

        Updates one or more negative keywords.

        Negative keywords submitted for update may have state set to pending for moderation review. Moderation may take up to 72 hours.
        Note that negative keywords can be updated on campaigns where serving status is not one of archived, terminated, rejected, or ended.
        Note that this operation supports a maximum list size of 100 negative keywords.

        Request Body
            | '**keywordId**': *number*, {'description': 'The identifier of the negative keyword.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which the negative keyword is associated.'}
            | '**campaignId**': *number*, {'description': 'The identifier of the campaign to which the negative keyword is associated.'}
            | '**state**': *string*, {'description': 'The current state of the negative keyword. Newly created SB negative keywords are in a default state of 'draft' before transitioning to a 'pending' state for moderation review. 'enabled' refers to negative keywords that are active. 'archived' refers to negative keywords that are permanently inactive and cannot be returned to the 'enabled' state.' , 'Enum': 'enabled, pending, archived, draft'}

        Returns:
            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/negativeKeywords', method='POST')
    def create_negative_keywords(self, **kwargs) -> ApiResponse:
        r"""
        Creates one or more negative keywords.

        Note that bid and state can't be set at negative keyword creation.

        Note that Negative keywords submitted for creation have state set to pending while under moderation review. Moderation review may take up to 72 hours.

        Note that negative keywords can be created on campaigns one where serving status is not one of archived, terminated, rejected, or ended.

        Note that this operation supports a maximum list size of 100 negative keywords.

        Request Body
            | '**campaignId**': *number*, {'description': 'The identifer of the campaign to which the keyword is associated.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which this keyword is associated.'}
            | '**state**': *string*, {'description': 'The current resource state.' , 'Enum': '[ enabled ]'}
            | '**keywordText**': *string*, {'description': 'The text of the expression to match against a search query.'}
            | '**matchType**': *string*, {'description': 'The type of match.' , 'Enum': '[ negativeExact, negativePhrase ]'}

        Returns:
            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/negativeKeywords/{}', method='GET')
    def get_negative_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        Gets a negative keyword specified by identifier.

        Keyword Args
            path **keywordId**:*number* | Required. The identifier of an existing keyword.


        Returns
            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/sb/negativeKeywords/{}', method='DELETE')
    def delete_negative_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        Archives a negative keyword specified by identifier.

        This operation is equivalent to an update operation that sets the status field to 'archived'. Note that setting the status field to 'archived' is permanent and can't be undone. See Developer Notes for more information.

        Keyword Args
            path **keywordId**:*number* | Required. The identifier of an existing keyword.

        Returns
            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)










from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Keywords(Client):

    @sp_endpoint('/v2/sp/keywords/{}', method='GET')
    def get_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        get_keyword(self, keywordId, **kwargs) -> ApiResponse

        Gets a keyword specified by identifier.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.
            query **locale**:*number* | Optional. The locale preference of the advertiser.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/keywords/{}', method='DELETE')
    def delete_keyword(self, keywordId, **kwargs) -> ApiResponse:
        r"""
        delete_keyword(self, keywordId, **kwargs) -> ApiResponse:

        Archives a keyword.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/keywords/extended/{}', method='GET')
    def get_keyword_extended(self, keywordId, **kwargs) -> ApiResponse:
        r"""

        get_keyword_extended(self, keywordId, **kwargs) -> ApiResponse

        Gets a keyword with extended data fields.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.
            query **locale**:*number* | Optional. The locale preference of the advertiser.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/keywords/extended', method='GET')
    def list_keywords_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_keywords_extended(self, **kwargs) -> ApiResponse

        Gets a list of keywords that have extended data fields.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list.. Available values : broad, phrase, exact.

            query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list..

            query **locale**:*string* | Optional. Restricts results to keywords associated with locale.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/keywords', method='GET')
    def list_keywords(self, **kwargs) -> ApiResponse:
        r"""
        list_keywords(self, **kwargs) -> ApiResponse

        Gets a list of keywords.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list.. Available values : broad, phrase, exact.

            query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

            query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.

            query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list..

            query **locale**:*string* | Optional. Restricts results to keywords associated with locale.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/sp/keywords', method='POST')
    def create_keywords(self, **kwargs) -> ApiResponse:
        r"""
        create_keywords(self, **kwargs) -> ApiResponse:

        Creates one or more keywords.

        body: | REQUIRED {'description': 'An array of keyword objects.}'

            | '**campaignId**': *number*, {'description': 'The identifer of the campaign to which the keyword is associated.'}
            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group to which this keyword is associated.'}
            | '**state**': *string*, {'description': 'The current resource state.' , 'Enum': '[ enabled, paused, archived ]'}
            | '**keywordText**': *string*, {'description': 'The keyword text.'}
            | '**nativeLanguageKeyword**': *string*, {'description': 'The unlocalized keyword text in the preferred locale of the advertiser.'}
            | '**nativeLanguageLocale**': *string*, {'description': 'The locale preference of the advertiser.'}
            | '**matchType**': *string*, {'description': 'The type of match.' , 'Enum': '[ exact, phrase, broad ]'}
            | '**bid**': *number($float)* {'description': 'Bid associated with this keyword. Applicable to biddable match types only.'}

        Returns:

            ApiResponse


        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/keywords', method='PUT')
    def edit_keywords(self, **kwargs) -> ApiResponse:
        r"""
        edit_keywords(self, **kwargs) -> ApiResponse:

        Updates one or more keywords.

        body: | REQUIRED {'description': 'An array of keyword objects.}'

            | '**keywordId**': *number*, {'description': 'The identifer of the campaign to which the keyword is associated.'}
            | '**state**': *string*, {'description': 'The current resource state.' , 'Enum': '[ enabled, paused, archived ]'}
            | '**bid**': *number($float)* {'description': 'Bid associated with this keyword. Applicable to biddable match types only.'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from setuptools import setup

setup(
    name='python-amazon-ad-api',
    version='0.0.9',
    install_requires=[
        "requests",
        "six",
        "cachetools~=4.2.2",
        "pycryptodome",
        "python-dotenv",
        "pytz",
        "confuse",
    ],
    packages=['ad_api','ad_api.api','ad_api.auth','ad_api.base','ad_api.api.sp','ad_api.api.sb'],
    url='https://github.com/denisneuf/python-amazon-ad-api',
    license='MIT',
    author='Daniel',
    author_email='info@leadtech.es',
    description='Python wrapper for the Amazon Advertising API'
)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Forecast(Client):

    @sp_endpoint('/sb/campaigns/shopperSegments/forecast', method='POST')
    def list_forecast(self, **kwargs) -> ApiResponse:
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Keywords(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/keywords', method='GET')
    def list_keywords(self, **kwargs) -> ApiResponse:
        r"""
        Gets an array of keywords, filtered by optional criteria.

        Keyword Args
            | query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0
            | query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.
            | query **matchTypeFilter**:*string* | Optional. Restricts results to keywords with match types within the specified comma-separated list.. Available values : broad, phrase, exact.
            | query **keywordText**:*string* | Optional. Restricts results to keywords that match the specified text exactly.
            | query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.
            | query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.
            | query **adGroupIdFilter**:*string* | Optional. Restricts results to keywords associated with ad groups specified by identifier in the comma-delimited list.
            | query **keywordIdFilter**:*string* | Optional. Restricts results to keywords associated with campaigns specified by identifier in the comma-delimited list..
            | query **creativeType**:*string* | Optional. Filter by the type of creative the campaign is associated with. To get ad groups associated with non-video campaigns specify 'productCollection'. To get ad groups associated with video campaigns, this must be set to 'video'. Returns all ad groups if not specified. Available values : productCollection, video
            | query **locale**:*string* | Optional. Restricts results to keywords associated with locale.

        Returns:
            | ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/sb/keywords', method='PUT')
    def edit_keywords(self, **kwargs) -> ApiResponse:
        """
        Updates one or more keywords.

        Keywords submitted for update may have state set to pending for moderation review. Moderation may take up to 72 hours.
        Note that keywords can be updated on campaigns where serving status is not one of archived, terminated, rejected, or ended.
        Note that this operation supports a maximum list size of 100 keywords.

        Request body
            | **keywordId** (integer($int64)): [required] The identifier of the keyword.
            | **adGroupId** (integer($int64)): [required] The identifier of an existing ad group to which the keyword is associated.
            | **campaignId** (integer($int64)) [required] The identifier of an existing campaign to which the keyword is associated.
            | **state** (string): [optional] Newly created SB keywords are in a default state of 'draft' before transitioning to a 'pending' state for moderation. After moderation, the keyword will be in an enabled state. Enum: [ enabled, paused, pending, archived, draft ]
            | **bid** (number) [optional] The bid associated with the keyword. Note that this value must be less than the budget associated with the Advertiser account. For more information, see the Keyword bid constraints by marketplace section of the supported features article

        Returns:
            | ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/keywords', method='POST')
    def create_keywords(self, **kwargs) -> ApiResponse:
        """
        Creates one or more keywords.

        Note that state can't be set at keyword creation. Keywords submitted for creation have state set to pending while under moderation review. Moderation review may take up to 72 hours.

        Note that keywords can be created on campaigns where serving status is not one of archived, terminated, rejected, or ended.

        Note that this operation supports a maximum list size of 100 keywords.

        Request body
            | **adGroupId** (integer($int64)) The identifier of an existing ad group to which the keyword is associated.
            | **campaignId** (integer($int64)) The identifier of an existing campaign to which the keyword is associated.
            | **keywordText** (string) The keyword text. The maximum number of words for this string is 10.
            | **nativeLanguageKeyword** (string) The unlocalized keyword text in the preferred locale of the advertiser.
            | **nativeLanguageLocale** (string) The locale preference of the advertiser. For example, if the advertiser’s preferred language is Simplified Chinese, set the locale to zh_CN. Supported locales include: Simplified Chinese (locale: zh_CN) for US, UK and CA. English (locale: en_GB) for DE, FR, IT and ES.
            | **matchType** (string) The match type. For more information, see match types in the Amazon Advertising support center. Enum: [ broad, exact, phrase ]
            | **bid** (number) [optional] The bid associated with the keyword. Note that this value must be less than the budget associated with the Advertiser account. For more information, see the Keyword bid constraints by marketplace section of the supported features article.

        Returns:
            | ApiResponse
        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/sb/keywords/{}', method='GET')
    def get_keyword(self, keywordId, **kwargs) -> ApiResponse:
        """
        Gets a keyword specified by identifier.

        Keyword Args
            | path **keywordId** (integer): The identifier of an existing keyword. [required]
            | query **locale** (string): The returned array includes only keywords associated with locale matching those specified by identifier. [optional]

        Returns:
            | ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/sb/keywords/{}', method='DELETE')
    def delete_keyword(self, keywordId, **kwargs) -> ApiResponse:
        """
        Archives a keyword specified by identifier.

        This operation is equivalent to an update operation that sets the status field to 'archived'. Note that setting the status field to 'archived' is permanent and can't be undone. See Developer Notes for more information.

        Keyword Args
            | path **keywordId** (integer): The identifier of an existing keyword. [required]


        Returns:
            | ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)











from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Campaigns(Client):

    @sp_endpoint('/v2/sp/campaigns', method='POST')
    def create_campaigns(self, **kwargs) -> ApiResponse:
        r"""
        create_campaigns(self, **kwargs) -> ApiResponse

        Creates one or more campaigns.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**portfolioId**': *number*, {'description': 'The identifier of an existing portfolio to which the campaign is associated'}
            | '**name**': *string*, {'description': 'A name for the campaign'}
            | '**tags**': *string*, {'description': 'A list of advertiser-specified custom identifiers for the campaign. Each customer identifier is a key-value pair. You can specify a maximum of 50 identifiers.'}
            | '**campaignType**': *string*, {'description': 'The advertising product managed by this campaign', 'Enum': '[ sponsoredProducts ]'}
            | '**targetingType**': *string*, {'description': 'The type of targeting for the campaign.', 'Enum': '[ manual, auto ]'}
            | '**state**': *string*, {'description': 'The current resource state.', 'Enum': '[ enabled, paused, archived ]'}
            | '**dailyBudget**': *number($float)*, {'description': 'A daily budget for the campaign.'}
            | '**startDate**': *string*, {'description': 'A starting date for the campaign to go live. The format of the date is YYYYMMDD.'}
            | '**endDate**': *string* nullable: true, {'description': 'An ending date for the campaign to stop running. The format of the date is YYYYMMDD.'}
            | '**premiumBidAdjustment**': *boolean*, {'description': 'If set to true, Amazon increases the default bid for ads that are eligible to appear in this placement. See developer notes for more information.'}
            | '**bidding**': *Bidding*, {'strategy': 'string', 'Enum': '[ legacyForSales, autoForSales, manual ]', 'adjustments': '{...}'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/campaigns', method='PUT')
    def edit_campaigns(self, **kwargs) -> ApiResponse:
        r"""
        edit_campaigns(self, **kwargs) -> ApiResponse

        Updates one or more campaigns.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**campaignId**': *number*, {'description': 'The identifier of an existing campaign to update.'}
            | '**portfolioId**': *number*, {'description': 'The identifier of an existing portfolio to which the campaign is associated'}
            | '**name**': *string*, {'description': 'The name for the campaign'}
            | '**tags**': *CampaignTags*, {'description': 'A list of advertiser-specified custom identifiers for the campaign. Each customer identifier is a key-value pair. You can specify a maximum of 50 identifiers.'}
            | '**state**': *string*, {'description': 'The current resource state.', 'Enum': '[ enabled, paused, archived ]'}
            | '**dailyBudget**': *number($float)*, {'description': 'The daily budget for the campaign.'}
            | '**startDate**': *string*, {'description': 'The starting date for the campaign to go live. The format of the date is YYYYMMDD.'}
            | '**endDate**': *string* nullable: true, {'description': 'The ending date for the campaign to stop running. The format of the date is YYYYMMDD.'}
            | '**premiumBidAdjustment**': *boolean*, {'description': 'If set to true, Amazon increases the default bid for ads that are eligible to appear in this placement. See developer notes for more information.'}
            | '**bidding**': *Bidding*, {'strategy': 'string', 'Enum': '[ legacyForSales, autoForSales, manual ]', 'adjustments': '{...}'}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/campaigns', method='GET')
    def list_campaigns(self, **kwargs) -> ApiResponse:
        r"""
        list_campaigns(self, **kwargs) -> ApiResponse

        Gets an array of campaigns.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **name**:*string* | Optional. Restricts results to campaigns with the specified name.

            query **portfolioIdFilter**:*string* | Optional. A comma-delimited list of portfolio identifiers.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/campaigns/{}', method='GET')
    def get_campaign(self, campaignId, **kwargs) -> ApiResponse:
        r"""

        get_campaign(self, campaignId, **kwargs) -> ApiResponse

        Gets a campaign specified by identifier.

            path **campaignId**:*number* | Required. The identifier of an existing campaign.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)

    @sp_endpoint('/v2/sp/campaigns/{}', method='DELETE')
    def delete_campaign(self, campaignId, **kwargs) -> ApiResponse:
        r"""

        delete_campaign(self, campaignId, **kwargs) -> ApiResponse

        Sets the campaign status to archived. Archived entities cannot be made active again. See developer notes for more information.

            path **campaignId**:*number* | Required. The identifier of an existing campaign.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)

    @sp_endpoint('/v2/sp/campaigns/extended', method='GET')
    def list_campaigns_extended(self, **kwargs) -> ApiResponse:
        r"""
        list_campaigns_extended(self, **kwargs) -> ApiResponse

        Gets an array of campaigns with extended data fields.

            query **startIndex**:*integer* | Optional. 0-indexed record offset for the result set. Default value : 0

            query **count**:*integer* | Optional. Number of records to include in the paged response. Defaults to max page size.

            query **stateFilter**:*string* | Optional. The returned array is filtered to include only ad groups with state set to one of the values in the specified comma-delimited list. Available values : enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived Default value : enabled, paused, archived.

            query **name**:*string* | Optional. Restricts results to campaigns with the specified name.

            query **portfolioIdFilter**:*string* | Optional. A comma-delimited list of portfolio identifiers.

            query **campaignIdFilter**:*string* | Optional. A comma-delimited list of campaign identifiers.

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/sp/campaigns/extended/{}', method='GET')
    def get_campaign_extended(self, campaignId, **kwargs) -> ApiResponse:
        r"""

        get_campaign_extended(self, campaignId, **kwargs) -> ApiResponse

        Gets an array of campaigns with extended data fields.

            path **campaignId**:*number* | Required. The identifier of an existing campaign.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), campaignId), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Portfolios(Client):
    """
    Portfolios consist of campaigns that are grouped together and linked to a distinct Advertiser Account. The term 'advertiser' refers to a brand, entity, account ID, or claim ID. An integrator can create multiple portfolios within an Advertiser Account.
    """

    @sp_endpoint('/v2/portfolios', method='GET')
    def list_portfolios(self, **kwargs) -> ApiResponse:
        r"""

        list_portfolios(self, **kwargs) -> ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/portfolios/extended', method='GET')
    def list_portfolios_extended(self, **kwargs) -> ApiResponse:
        r"""

        list_portfolios_extended(self, **kwargs) -> ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/v2/portfolios/{}', method='GET')
    def get_portfolio(self, portfolioId, **kwargs) -> ApiResponse:
        r"""

        get_portfolio(self, portfolioId, **kwargs) -> ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), portfolioId), params=kwargs)

    @sp_endpoint('/v2/portfolios/extended/{}', method='GET')
    def get_portfolio_extended(self, portfolioId, **kwargs) -> ApiResponse:
        r"""

        get_portfolio_extended(self, portfolioId, **kwargs) -> ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), portfolioId), params=kwargs)

    @sp_endpoint('/v2/portfolios', method='POST')
    def create_portfolios(self, **kwargs) -> ApiResponse:
        r"""

        create_portfolios(self, **kwargs) -> ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/portfolios', method='PUT')
    def edit_portfolios(self, **kwargs) -> ApiResponse:
        r"""

        edit_portfolios(self, **kwargs) -> ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class BidRecommendations(Client):

    @sp_endpoint('/v2/sp/adGroups/{}/bidRecommendations', method='GET')
    def get_ad_group_bid_recommendations(self, adGroupId, **kwargs) -> ApiResponse:
        r"""
        get_ad_group_bid_recommendations(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets a bid recommendation for an ad group.

            path **adGroupId**:*number* | Required. The identifier of an existing ad group.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), adGroupId), params=kwargs)


    @sp_endpoint('/v2/sp/keywords/{}/bidRecommendations', method='GET')
    def get_keyword_bid_recommendations(self, keywordId, **kwargs) -> ApiResponse:
        r"""
        get_ad_group_bid_recommendations(self, adGroupId, \*\*kwargs) -> ApiResponse

        Gets a bid recommendation for a keyword.

            path **keywordId**:*number* | Required. The identifier of an existing keyword.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), keywordId), params=kwargs)

    @sp_endpoint('/v2/sp/keywords/bidRecommendations', method='POST')
    def get_keywords_bid_recommendations(self, **kwargs) -> ApiResponse:
        r"""
        get_keywords_bid_recommendations(self, \*\*kwargs) -> ApiResponse:

        Gets bid recommendations for keywords.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**adGroupId**': *number*, {'description': 'The identifier of the ad group.'}
            | keywords {
            | '**keywords**': *string*, {'description': 'The keyword text.'}
            | '**matchType**': *string*, {'description': 'The type of match', 'Enum': '[ exact, phrase, broad ]'}
            | }


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/sp/targets/bidRecommendations', method='POST')
    def get_targets_bid_recommendations(self, **kwargs) -> ApiResponse:
        r"""
        get_targets_bid_recommendations(self, \*\*kwargs) -> ApiResponse:

        Gets a list of bid recommendations for keyword, product, or auto targeting expressions.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**adGroupId**': *number*, {'description': 'The ad group identifier.'}
            | expressions {
            | '**value**': *string*, {'description': 'The expression value.'}
            | '**type**': *string*, {'description': 'The type of targeting expression', 'Enum': '[ queryBroadMatches, queryPhraseMatches, queryExactMatches, asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs, queryBroadRelMatches, queryHighRelMatches, asinSubstituteRelated, asinAccessoryRelated, asinAgeRangeSameAs, asinGenreSameAs, asinIsPrimeShippingEligible ]'}
            | }


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)


import sys
import os
import logging
from enum import Enum
from dotenv import dotenv_values

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s:%(levelname)s:%(message)s"
)

class AuthorizationError(Exception):
    def __init__(self, code, message=".env problem set your file in the root project"):
        logging.warning(message);

class AWS_ENV(Enum):
    PRODUCTION = "PRODUCTION"
    SANDBOX = "SANDBOX"


class Marketplaces(Enum):
    NA = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api.amazon.com',
           'currency': 'USD',
           'token_url': 'api.amazon.com/auth/o2/token'}

    EU = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'EUR',
           'token_url': 'api.amazon.com/auth/o2/token'}

    GB = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'GBP',
           'token_url': 'api.amazon.com/auth/o2/token'}

    ES = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'EUR',
           'token_url': 'api.amazon.com/auth/o2/token'}

    DE = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'EUR',
           'token_url': 'api.amazon.com/auth/o2/token'}

    IT = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'EUR',
           'token_url': 'api.amazon.com/auth/o2/token'}

    FR = {'sandbox': 'advertising-api-test.amazon.com',
           'prod': 'advertising-api-eu.amazon.com',
           'currency': 'EUR',
           'token_url': 'api.amazon.com/auth/o2/token'}

    def __init__(self, info):

        config = dotenv_values(".env")
        AWS_ENVIRONMENT = config.get('AWS_ENV') or os.environ.get('API_PASSWORD')
        if AWS_ENVIRONMENT=="PRODUCTION":
            self.region_url = info.get('prod')
        else:
            self.region_url = info.get('sandbox')

        self.endpoint = 'https://{}'.format(self.region_url)
        self.currency = info.get('currency')



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Invoices(Client):

    @sp_endpoint('/invoices', method='GET')
    def list_invoices(self, **kwargs) -> ApiResponse:
        r"""

        list_invoices(self, **kwargs) -> ApiResponse

        Get invoices for advertiser. Requires one of these permissions: ["nemo_transactions_view","nemo_transactions_edit"]

            query **invoiceStatuses**:*string* | Optional. Available values : ISSUED, PAID_IN_PART, PAID_IN_FULL, WRITTEN_OFF.
            query **count**:*string* | Optional. Number of records to include in the paged response. Defaults to 100. Cannot be combined with the cursor parameter.
            query **cursor**:*string* | Optional. A cursor representing how far into a result set this query should begin. In the absence of a cursor the request will default to start index of 0 and page size of 100.


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'),  params=kwargs)

    @sp_endpoint('/invoices/{}', method='GET')
    def get_invoice(self, invoiceId, **kwargs) -> ApiResponse:
        r"""

        get_invoice(self, invoiceId, **kwargs) -> ApiResponse

        Get invoice data by invoice ID. Requires one of these permissions: ["nemo_transactions_view","nemo_transactions_edit"]

            path **invoiceId**:*string* | Optional. Available values : ISSUED, PAID_IN_PART, PAID_IN_FULL, WRITTEN_OFF.

        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), invoiceId), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class BidRecommendations(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/sb/recommendations/bids', method='POST')
    def get_bid_recommendations(self, **kwargs) -> ApiResponse:
        r"""
        Get a list of bid recommendation objects for a specified list of keywords or products.

        Request Body
            | '**campaignId**': *integer($int64)*, {'description': 'The identifier of the campaign for which bid recommendations are created.'}
            | '**targets**': *integer($int64)*, {'SBExpression': 'A name value pair that defines a targeting expression. The type field defines the predicate. The value field defines the value to match for the predicate.'}

                | '**type**': *string*, {'values': '[ asinCategorySameAs, asinBrandSameAs, asinPriceLessThan, asinPriceBetween, asinPriceGreaterThan, asinReviewRatingLessThan, asinReviewRatingBetween, asinReviewRatingGreaterThan, asinSameAs ]'}

                | '**value**': *string*, {'description': 'The text of the targeting expression. The - token defines a range. For example, 2-4 defines a range of 2, 3, and 4.'}

            | '**keywords**': *string*, {'description': 'SBBidRecommendationKeyword'}

                | '**matchType**': *string*, {'values': '[ broad, exact, phrase ]'}

                | '**keywordText**': *string*, {'description': 'The text of the keyword. Maximum of 10 words.'}

            | '**adFormat**': *integer($int64)*, {'values': '[ productCollection, video ]'}

        Returns:
            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Stores(Client):
    """
    Use the Amazon Advertising API for Sponsored Brands for campaign, ad group, keyword, negative keyword, drafts, Stores, landing pages, and Brands management operations. For more information about Sponsored Brands, see the Sponsored Brands Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/stores/assets', method='GET')
    def list_assets(self, **kwargs) -> ApiResponse:
        """
        For sellers or vendors, gets an array of assets associated with the specified brand entity identifier. Vendors are not required to specify a brand entity identifier, and in this case all assets associated with the vendor are returned.

        Keyword Args
            | path **brandEntityId** (string): For sellers, this field is required. It is the Brand entity identifier of the Brand for which assets are returned. This identifier is retrieved using the getBrands operation. For vendors, this field is optional. If a vendor does not specify this field, all assets associated with the vendor are returned. For more information about the difference between a seller and a vendor, see the Amazon Advertising FAQ. [required]

            | query **mediaType** (string): Specifies the media types used to filter the returned array. Currently, only the brandLogo type is supported. If not specified, all media types are returned. Available values : brandLogo, image [optional]

        Returns:
            | ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path')), params=kwargs)


    @sp_endpoint('/v2/stores', method='GET')
    def list_stores(self, **kwargs) -> ApiResponse:
        """
        List store information for all registered stores under an advertiser.

        Keyword Args
            None

        """
        return self._request(fill_query_params(kwargs.pop('path')), params=kwargs)

    # It is not working the endpoint
    @sp_endpoint('/v2/stores/{}', method='GET')
    def get_store(self, brandEntityId, **kwargs) -> ApiResponse:
        return self._request(fill_query_params(kwargs.pop('path'), brandEntityId), params=kwargs)

    # It is not working Cannot consume content type
    @sp_endpoint('/stores/assets', method='POST')
    def create_asset(self, contentDisposition, contentType, **kwargs) -> ApiResponse:
        headers = {'Content-Disposition': contentDisposition, 'Content-Type': contentType}
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs, headers=headers)


from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Profiles(Client):
    """
    Profiles AD-API Client
    :link:
    With the Profiles.
    """

    @sp_endpoint('/v2/profiles', method='GET')
    def list_profiles(self, **kwargs) -> ApiResponse:
        r"""

        list_profiles(self, **kwargs) -> ApiResponse

        Gets a list of profiles.

            query **apiProgram**:*string* | Optional. Filters response to include profiles that have permissions for the specified Advertising API program only. Available values : billing, campaign, paymentMethod, store, report, account, posts

            query **accessLevel**:*string* | Optional. Filters response to include profiles that have specified permissions for the specified Advertising API program only. Available values : edit, view

            query **profileTypeFilter**:*string* | Optional. Filters response to include profiles that are of the specified types in the comma-delimited list. Available values : seller, vendor, agency

            query **validPaymentMethodFilter**:*string* | Optional. Filter response to include profiles that have valid payment methods. Available values : true, false


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), params=kwargs)

    @sp_endpoint('/v2/profiles', method='PUT')
    def update_profile(self, **kwargs) -> ApiResponse:
        r"""

        update_profile(self, **kwargs) -> ApiResponse

        Update the daily budget for one or more profiles. Note that this operation is only used for Sellers using Sponsored Products.

        body: | REQUIRED {'description': 'An array of ad groups.}'

            | '**profileId**': *integer($int64)*, {'description': 'The identifier of the profile.'}
            | '**countryCode**': *string*, {'description': 'The countryCode for a given country'}
            | '**currencyCode**': *string*, {'description': 'The currency used for all monetary values for entities under this profile.'}
            | '**dailyBudget**': *number*, {'description': 'Note that this field applies to Sponsored Product campaigns for seller type accounts only. Not supported for vendor type accounts.'}
            | '**timezone**': *string*, {'description': 'The time zone used for all date-based campaign management and reporting.'}
            | '**accountInfo**': *AccountInfoAccountInfo*, {}

        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/profiles/{}', method='GET')
    def get_profile(self, profileId, **kwargs) -> ApiResponse:
        r"""

        get_profile(self, profileId, **kwargs) -> ApiResponse

        Gets a profile specified by identifier.

            path **profileId**:*number* | Required. The identifier of an existing profile Id.


        Returns:

            ApiResponse

        """
        return self._request(fill_query_params(kwargs.pop('path'), profileId), params=kwargs)


    @sp_endpoint('/v2/profiles/registerBrand', method='PUT')
    def register_brand(self, **kwargs) -> ApiResponse:
        r"""

        register_brand(self, **kwargs) -> ApiResponse

        SANDBOX ONLY - Create a vendor profile for sandbox.

        body: | REQUIRED

            '**countryCode**': *string*, {'description': 'The countryCode for a given country'}
            '**brand**': *string*, {'description': 'The brand for the vendor account'}


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/profiles/register', method='PUT')
    def register(self, **kwargs) -> ApiResponse:
        r"""

        register_brand(self, \*\*kwargs) -> ApiResponse

        SANDBOX ONLY - Create a seller profile for sandbox.

        body: REQUIRED

            '**countryCode**': *string*, {'description': 'The countryCode for a given country'}


        Returns:

            ApiResponse

        """
        return self._request(kwargs.pop('path'), data=kwargs.pop('body'), params=kwargs)



from ad_api.base import Client, sp_endpoint, fill_query_params, ApiResponse

class Snapshots(Client):
    """
    Use the Amazon Advertising API for Sponsored Products for campaign, ad group, keyword, negative keyword, and product ad management operations. For more information about Sponsored Products, see the Sponsored Products Support Center. For onboarding information, see the account setup topic.
    """
    @sp_endpoint('/v2/sp/{}/snapshot', method='POST')
    def post_snapshot(self, recordType, **kwargs) -> ApiResponse:
        """
        Request a file-based snapshot of all entities of the specified type in the account satisfying the filtering criteria.

        Keyword Args
            | path **recordType** (integer): The type of entity for which the snapshot is generated. Available values : campaigns, adGroups, keywords, negativeKeywords, campaignNegativeKeywords, productAds, targets, negativeTargets [required]

        Request body
            | **stateFilter** (string): [required] [ enabled, paused, archived, enabled, paused, enabled, archived, paused, archived, enabled, paused, archived ].

        Returns:
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), recordType), data=kwargs.pop('body'), params=kwargs)

    @sp_endpoint('/v2/snapshots/{}', method='GET')
    def get_snapshot(self, snapshotId, **kwargs) -> ApiResponse:
        r"""
        Gets the status of a requested snapshot.

        Keyword Args
            | path **snapshotId** (number): The snapshot identifier. [required]

        Returns:
            ApiResponse
        """
        return self._request(fill_query_params(kwargs.pop('path'), snapshotId), params=kwargs)

    def download_snapshot(self, **kwargs) -> ApiResponse:
        r"""
        Downloads the snapshot previously get report specified by location (this is not part of the official Amazon Advertising API, is a helper method to download the snapshot). Take in mind that a direct download of location returned in get_snapshot will return 401 - Unauthorized.

        kwarg parameter **file** if not provided will take the default amazon name from path download (add a path with slash / if you want a specific folder, do not add extension as the return will provide the right extension based on format choosed if needed)

        kwarg parameter **format** if not provided a format will return a url to download the snapshot (this url has a expiration time)

        Keyword Args
            | **url** (string): The location obatined from get_snapshot [required]
            | **file** (string): The path to save the file if mode is download json, zip or gzip. [optional]
            | **format** (string): The mode to download the snapshot: data (list), raw, url, json, zip, gzip. Default (url) [optional]

        Returns:
            ApiResponse
        """
        return self._download(self, params=kwargs)